import 'package:flutter/material.dart';
import 'package:mini_calendar/handle.dart';
import 'package:mini_calendar/mini_calendar.dart';
import 'package:mini_calendar/model/date_day.dart';

import 'month_widget.dart';

///
class MiniCalender<T> extends StatefulWidget {
  /// 当前选择的日期
  final DateDay currentDay;

  /// 当前选中的月份
  final DateDay currentMonth;

  /// 最小日期
  final DateDay minTime;

  /// 最大日期
  final DateDay maxTime;

  /// 日控件高度
  final double dayHeight;

  /// 日历宽度
  final double width;

  /// 月份滑动改变
  final ValueChanged<DateDay> onMonthChange;

  /// 显示星期
  final bool showWeekHeader;

  /// 标记集合
  final Map<DateDay, T> marks;

  final BuildMark<T> buildMark;

  /// 日期选择事件
  final OnDaySelected<T> onDaySelected;
  final _MiniCalenderState _state = _MiniCalenderState();

  MiniCalender({
    Key key,
    this.currentDay,
    this.minTime,
    this.maxTime,
    this.onMonthChange,
    this.dayHeight,
    this.width = double.infinity,
    this.showWeekHeader = true,
    this.marks = const {},
    this.buildMark,
    this.onDaySelected,
    this.currentMonth,
  }) : super(key: key);

  @override
  _MiniCalenderState createState() => _state;

  void next() => _state._next();

  bool prev() => _state._prev();

  bool goToDay(DateDay day) => _state._goToDay(day);
}

class _MiniCalenderState extends State<MiniCalender> {
  List<DateDay> months = [];
  PageController _controller;
  int _leftEndPosition = 1;
  int _rightEndPosition = 1;
  DateDay _currentDay;
  DateDay _currentMonth;

  int _position;

  bool _goToDay(DateDay day) {
    for (int index = 0; index < months.length; index++) {
      if (months[index].year == day.year && months[index].month == day.month) {
        _position = index;
        _controller.jumpToPage(_position++);
        return true;
      }
    }
    return false;
  }

  bool _next() {
    if (_position == (_leftEndPosition + _rightEndPosition)) return false;
    _controller.animateToPage(++_position, duration: const Duration(milliseconds: 500), curve: Curves.ease);
    return true;
  }

  bool _prev() {
    if (_position == 0) return false;
    _controller.animateToPage(--_position, duration: const Duration(milliseconds: 500), curve: Curves.ease);
    return true;
  }

  @override
  void initState() {
    DateDay _startTime = widget.minTime;
    DateDay _endTime = widget.maxTime;
    _currentDay = widget.currentDay != null ? widget.currentDay.copyWith() : DateDay.now();
    _currentMonth = widget.currentMonth != null ? widget.currentMonth.copyWith() : _currentDay.copyWith();

    if (_startTime == null || _startTime > _currentDay) {
      _startTime = _currentDay.monthFirstDay;
    }
    if (_endTime == null || _endTime < _currentDay) {
      _endTime = _currentDay.monthEndDay;
    }
    _leftEndPosition = (_currentMonth.year - _startTime.year) * 12 + _currentMonth.month - _startTime.month;
    _position = _leftEndPosition;
    _rightEndPosition = (_endTime.year - _currentMonth.year) * 12 + _endTime.month - _startTime.month;

    List.generate(_leftEndPosition, (index) {
      months.add(_currentMonth.copyWith(month: _currentMonth.month - _leftEndPosition + index, day: 1));
    });
    months.add(_currentMonth.copyWith());
    List.generate(_rightEndPosition, (index) {
      months.add(_currentMonth.copyWith(month: _currentMonth.month + index + 1, day: 1));
    });
    _controller = PageController(initialPage: _position);
    super.initState();
  }

  @override
  void dispose() {
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    PageView pageView = PageView(
      controller: _controller,
      children: months
          .map(
            (month) => MonthWidget(
              inMonth: month,
              currentDay: _currentDay,
              dayHeight: widget.dayHeight,
              width: widget.width,
              buildMark: widget.buildMark,
              marks: widget.marks,
              onDaySelected: (day, data) {
                _currentDay = day.copyWith();
                setState(() {});
                if (widget.onDaySelected != null) widget.onDaySelected(_currentDay, data);
              },
            ),
          )
          .toList(),
      onPageChanged: (position) {
        this._position = position;
        _currentMonth = months[this._position].copyWith();
        if (widget.onMonthChange != null) widget.onMonthChange(_currentMonth);
      },
    );

    return widget.showWeekHeader
        ? Column(
            children: <Widget>[_buildWeekHeader(), Expanded(child: pageView)],
          )
        : pageView;
  }

  Widget _buildWeekHeader() {
    return Padding(
      padding: EdgeInsets.only(top: 15, bottom: 10, left: 5, right: 5),
      child: Row(
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: <Widget>[
            Text('星期一', style: defaultDayStyle.copyWith(color: Colors.black)),
            Text('星期二', style: defaultDayStyle.copyWith(color: Colors.black)),
            Text('星期三', style: defaultDayStyle.copyWith(color: Colors.black)),
            Text('星期四', style: defaultDayStyle.copyWith(color: Colors.black)),
            Text('星期五', style: defaultDayStyle.copyWith(color: Colors.black)),
            Text('星期六', style: defaultDayStyle.copyWith(color: Colors.pink[300])),
            Text('星期日', style: defaultDayStyle.copyWith(color: Colors.pink[300])),
          ]),
    );
  }
}
